<?php

namespace Slovocast\Infrastructure\Session;

use Odan\Session\SessionInterface;
use Odan\Session\FlashInterface;

class FlashMessages implements FlashInterface
{
    const LEVEL_ERROR = "error";
    const LEVEL_NOTICE = "notice";
    const LEVEL_SUCCESS = "success";

    private string $key = "_flash";

    public function __construct(
        protected SessionInterface $session
    ) {
        $this->session->set($this->key, []);
    }

    /**
     * @return array<string, array<string>>
     */
    protected function getMessages(): array
    {
        return $this->session->get($this->key);
    }

    /**
     * @param array<string, array<string>> $messages
     * @return void
     */
    protected function saveMessages(array $messages): void
    {
        $this->session->set($this->key, $messages);
    }

    public function has(string $key): bool 
    {
        return array_key_exists($key, $this->getMessages());
    }
    
    public function set(string $key, array $messages): void
    {
        $messages = $this->getMessages();
        $messages[$key] = $messages;
        $this->saveMessages($messages);
    }

    public function add(string $key, string $message): void
    {
        $messages = $this->getMessages();
        $messages[$key][] = $message;
        $this->saveMessages($messages);
    }

    public function get(string $level): array
    {
        $messages = $this->getMessages();

        if (!array_key_exists($level, $messages)) {
            return [];
        }

        $buffer = $messages[$level];
        unset($messages[$level]);
        $this->saveMessages($messages);
        return $buffer;
    }

    public function all(): array
    {
        $messages = $this->getMessages();
        $this->clear();
        return $messages;
    }

    public function clear(): void
    {
        $this->session->set($this->key, []);
    }
}
